/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bespin.
 *
 * The Initial Developer of the Original Code is
 * Mozilla.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Bespin Team (bespin@mozilla.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var catalog = require("bespin:plugins").catalog;
var Promise = require("bespin:promise").Promise;

var r = require;

/**
 * Convert some data from a string to another type as specified by
 * <tt>typeSpec</tt>.
 */
exports.fromString = function(stringVersion, typeSpec) {
    var promise = new Promise();

    exports.getTypeExt(typeSpec).then(function(typeExt) {
        typeExt.load(function(type) {
            var objectVersion = type.fromString(stringVersion, typeExt);
            promise.resolve(objectVersion);
        });
    });

    return promise;
};

/**
 * Convert some data from an original type to a string as specified by
 * <tt>typeSpec</tt>.
 */
exports.toString = function(objectVersion, typeSpec) {
    var promise = new Promise();

    exports.getTypeExt(typeSpec).then(function(typeExt) {
        typeExt.load(function(type) {
            var stringVersion = type.toString(objectVersion, typeExt);
            promise.resolve(stringVersion);
        });
    });

    return promise;
};

/**
 * Convert some data from an original type to a string as specified by
 * <tt>typeSpec</tt>.
 */
exports.isValid = function(originalVersion, typeSpec) {
    var promise = new Promise();

    exports.getTypeExt(typeSpec).then(function(typeExt) {
        typeExt.load(function(type) {
            var valid = type.isValid(originalVersion, typeExt);
            promise.resolve(valid);
        });
    });

    return promise;
};

/**
 * typeSpec one of:
 * "typename",
 * "typename:json" e.g. 'selection:["one", "two", "three"]'
 * { name:"typename", data:... } e.g. { name:"selection", data:["one", "two", "three"] }
 */
exports.getTypeExt = function(typeSpec) {
    var promise = new Promise();

    var typeExt;
    if (typeof typeSpec === "string") {
        var parts = typeSpec.split(":");
        if (parts.length === 1) {
            typeExt = catalog.getExtensionByKey("type", typeSpec);
            if (typeExt) {
                promise.resolve(typeExt);
            } else {
                promise.reject("Unknown type: " + typeSpec);
            }
        } else {
            typeExt = catalog.getExtensionByKey("type", parts.shift());
            var data = parts.join(":");
            if (data.substring(0, 1) == "[" || data.substring(0, 1) == "{") {
                typeExt.data = JSON.parse(data);
                promise.resolve(typeExt);
            } else {
                var parts = data.split("#");
                var modName = parts.shift();
                var objName = parts.join("#");

                r.loader.async(modName).then(function() {
                    var module = r(modName);
                    typeExt.data = module[objName]();
                    promise.resolve(typeExt);
                });
            }
        }
    } else if (typeof typeSpec === "object") {
        typeExt = catalog.getExtensionByKey("type", typeSpec.name);
        typeExt.data = typeSpec.data;
        promise.resolve(typeExt);
    }

    return promise;
};
